import cherrypy
import os, glob
import subway
from subway.framework import importing

try:
    import sqlobject
    from sqlobject import dbconnection
    sqlhub = dbconnection.ConnectionHub()
    class SQLModel(sqlobject.SQLObject):
        _connection = sqlhub
        fields = property(lambda self: dict(self.field_list))
        field_list = property(lambda self: [(col.name, getattr(self, col.name)) for col in self._columns])
    def sql_connect(index):
        """
        give sqlobject access in this thread
        """
        if getattr(sqlhub, "threadConnection", None):
            return
        uri = subway.config_get("subway.sql_connection_uri", None)
        if uri:
            sqlhub.threadConnection = sqlobject.connectionForURI(uri)
            if uri.startswith("sqlite"):
                sqlobject.dbconnection.TheURIOpener.cachedURIs = {}

    def model_loader(index):
        """
        Import and setup models in subway.model
        """
        model_dir = os.path.join(subway.get_app_dir("model"))
        model_files = glob.glob(os.path.join(model_dir, "*.py"))
        subway.model.all_models = importing.import_models(model_files)
        
    cherrypy.server.onStartThreadList += [sql_connect, model_loader]
except ImportError:
    cherrypy.log("SQLModel unavailable; please install SQLObject.", severity=1)
    SQLModel = object
    sqlhub = None
    sql_connect = lambda _: None

class ModelManager:
    """
    Let import models in starting threads, so model operations (like creating tables)
    goes in the right connection thread.
    """
    def __init__(self):
        self.all_models = {}
    def __getattr__(self, attr):
        try:
            return self.all_models[attr]
        except KeyError:
            raise AttributeError, attr

