from cherrypy.lib.filter import basefilter
import sys, os
from subway import view, api
import cherrypy, traceback
from cherrypy import _cputil

class ErrorFilter(basefilter.BaseFilter):
    def __init__(self):
        self.errors = view.ViewManager(os.path.join(api.get_app_dir("view"),
                                                    "error"), api.get_app_dir("view"))

    def _render_error(self, exc_info, status):
        response = cherrypy.response
        exc_type = exc_info[0]
        exc_value = exc_info[1]
        exc_traceback = exc_info[2]
        formatted_tb = traceback.format_exception(*exc_info)
        formatted_tb ="<hr/>".join(formatted_tb)
        formatted_tb = formatted_tb.replace("\n", "<br />\n")
        try:
            response.body = _cputil.getSpecialAttribute("_error_%s"
                                                        % status)(exc_type, exc_value, exc_traceback, formatted_tb)
        except:
            response.body = self.errors.render("page%s.html" %
                                               status)
        
    def _handle_error(self):
        exc_info = sys.exc_info()
        if exc_info[0]:
            response = cherrypy.response
            if not response.status:
                # If not set, it's an internal error (see _cputil._cpOnError)
                response.status = 500
            elif response.status < 400: 
                # Let non-errors pass 
                return

            try:
                self._render_error(exc_info, str(response.status))
            except view.ViewNotFound:
                # No error renders have been found
                pass
            except:
                cherrypy.log(str(sys.exc_value))

    beforeFinalize = _handle_error
    beforeErrorResponse = _handle_error
