import sys, os
import cherrypy
from cherrypy.lib import cptools
from cStringIO import StringIO
from unittest import TestCase as _TestCase
import imp
from subway.framework import loader
from subway import skeleton
import subway
from sqlobject.declarative import DeclarativeMeta

view=None
current_testcase = None
orig_homedir = None
orig_viewdir = None
class test_cases:
    pass

cherrypy.server.onStartThreadList = []

class TestCase(_TestCase):
    def start_app(self, root, force=False, homedir="", viewdir="", config={}):
        """
        Initialize the Subway app
        """
        global current_testcase
        if current_testcase != self.__class__ or (cherrypy.server.state and force):
            cherrypy.server.stop()
            current_testcase = self.__class__

        if cherrypy.server.state == 0:
            if not homedir:
                if orig_homedir:
                    homedir = orig_homedir
                else:
                    homedir = os.path.dirname(skeleton.__file__)
                
            if not viewdir:
                if orig_viewdir:
                    viewdir = orig_viewdir
                else:
                    viewdir = os.path.join(test_cases.__path__, 'view')

            cherrypy.config.reset()
            loader.init_app(homedir)
            root.__file__ = homedir # get_app_dir() needs it
            cherrypy.root = subway.Controller(root)
            cherrypy.config.update({"server.environment": "production",
                                    "server.logToScreen": False},
                                   )
            cherrypy.config.update(config)
            global view
            view = subway.ViewManager(viewdir, viewdir)
            cherrypy.server.start(initOnly=True, serverClass=None)
            subway.datamodel.sql_connect(-1) # Force SQLObject connection

    def stop_app(self):
        cherrypy.server.stop()

    def load_models(self, autocreate=True, force=False, namespace={}):
        """
        Load models in subway.model.
        namespace defaults to sys._getframe(1).f_globals
        """
        if subway.model.all_models and not force:
            return
        if not namespace:
            namespace = sys._getframe(1).f_globals
        for objname, obj in namespace.items():
            if isinstance(obj, DeclarativeMeta):
                subway.model.all_models[objname] = obj
                if autocreate:
                    obj.createTable(ifNotExists=True)

    def request(self, page, method="GET", data="", headers=(), client_address="127.0.0.1", remote_host="localhost", scheme="http", protocol="HTTP/1.0"):
        """
        Make a request and return the response
        """
        if isinstance(headers, dict):
            headers = zip(dict.keys(), dict.values)
        rfile = StringIO(data)
        cherrypy.request.purge__()
        cherrypy.response.purge__()
        request_line = " ".join([method, page, protocol])
        cherrypy.server.request(client_address, remote_host, request_line, headers, rfile, scheme)
        return cherrypy.response

def validate_form(source, *args):
    return subway.validate_form(view.form(source), *args)

class Controller(cptools.PositionalParametersAware):
    """
    Base Controller class.
    """

    @subway.expose
    def default(self, *args, **kwargs):
        return cptools.PositionalParametersAware.default(self, *args, **kwargs)

    def _cpOnError(self):
        body = cherrypy.response.body
        _cputil._cpOnError()
        if body:
            cherrypy.response.body = body

def import_tests(path=os.getcwd(), prefix="test", homedir=None, viewdir=None):
    """
    Automatically import tests
    path - Path where to search for Python tests
    prefix - Python files should start with the given prefix
    """
    global orig_homedir, orig_viewdir
    orig_homedir = homedir
    orig_viewdir = viewdir
    test_cases.__path__ = path
    
    dir_content = os.listdir(path)
    for filename in dir_content:
        if filename.startswith(prefix) and filename.endswith(".py") and os.path.isfile(os.path.join(path, filename)):
            module_name = os.path.splitext(filename)[0]
            module = imp.load_module(module_name, *(imp.find_module(module_name, [path])))
            for objname in dir(module):
                obj = getattr(module, objname)
                try:
                    if issubclass(obj, TestCase):
                        setattr(test_cases, obj.__name__, obj)
                except:
                    # obj is not a class
                    pass
    return test_cases

from subway.cherryflow import common

from subway import util
from cherrypy.lib import cptools

def _flow(method):
    """
    Stolen from subway.cherryflow.standard and modified for class-use
    """
    flowid = common._get_flowid()

    def _m(self, _resume=0, *args, **kwargs):
        # this is the actual cherrypy handler function
        # _resume is the magic variable which lets the server know
        # at which point in the flow the client is resuming.
        #util.no_cache()
        try:
            _resume = int(_resume)
        except:
            if method.func_name == 'index':
                raise TypeError
            raise
        gen = common._get_gen(flowid, _resume, method, self)
        result = gen.next() # execute the next resume point
        if result == common.MILESTONE:
            common.save_milestone(flowid, _resume)
            result = gen.next()
        common._save_resumepoint(flowid, gen)
        return result # and return the response
    #return rewrite.flow_rewrite(_m)
    _m.__doc__ = method.__doc__
    return _m

flow = lambda func: cptools.decorate(func, _flow)
