import subway, cherrypy
from subway.framework import test

class Subdir(test.Controller):
    @subway.expose
    def index(self):
        return "sub controller"

class Root(test.Controller):
    subdir = Subdir()

    @subway.expose
    def index(self):
        return "hello tests!"

    @subway.expose
    def error(self):
        raise UserError

    @subway.expose
    def url(self, *args, **kwargs):
        return str(args)+str(kwargs)

    @subway.expose("aliastest")
    def alias(self):
        return ['aliased']

    def _error_404(self, *args, **kwargs):
        return "not found"

    def _error_500(self, *args, **kwargs):
        return "a"*550

class TestBasic(test.TestCase):
    def setUp(self):
        self.start_app(Root())
        
    def test_basic_request(self):
        self.assertEqual(self.request("/").body, ['hello tests!'])

    def test_error_404(self):
        res = self.request("/notfound")
        self.assertEqual(res.headerMap['Content-Length'], 513)
        self.assertEqual(res.body, ['not found'+" "*(513-len('not found'))])
        self.assertEqual(res.status, "404 Not Found")

    def test_error_500(self):
        res = self.request("/error")
        self.assertEqual(res.headerMap['Content-Length'], 550)
        self.assertEqual(res.body, ["a"*550])

    def test_no_expose(self):
        self.assertEqual(self.request("/_error_404").status, "404 Not Found")
        self.assertEqual(self.request("/_error_500").status, "404 Not Found")

    def test_sub_controllers(self):
        self.assertEqual(self.request("/subdir/").body, ["sub controller"])

    def test_virtual_path(self):
        self.start_app(Root(), force=True, config={'subway.virtual_path': '/virtual'})
        self.assertEqual(self.request("/virtual/").body, ["hello tests!"])
        self.stop_app()

    def test_url(self):
        self.assertEqual(self.request("/url/1/2/3?a=x&b=y").body, ["('1', '2', '3'){'a': 'x', 'b': 'y'}"])

    def test_aliases(self):
        self.assertEqual(self.request("/aliastest").body, ['aliased'])
