from subway import framework

def get_homedir():
    """
    Get homedir by parsing the caller module __file__
    """
    import os, sys
    # Frame 1 is wsgi_server(), frame 2 is the wsgi_server() caller module
    return os.path.split(os.path.dirname(os.path.abspath(sys._getframe(2).f_globals['__file__'])))[0]
    
def prepare_app(homedir, daemons="*", main_thread=False):
    """
    Initialize application and load daemons
    """
    framework.init_app(homedir)
    if daemons == "*":
        daemons = None
    elif daemons == "none":
        daemons = []
    else:
        daemons = daemons.split(",")
    framework.load_daemons(homedir, daemons, main_thread)

def start_server(homedir=None):
    """
    start_server(): execute a Subway app
    assumes homedir is one level up from caller module path unless
    argument is provided.
    """
    import optparse
    if not homedir:
        homedir = get_homedir()
        
    parser = optparse.OptionParser()
    parser.add_option("-w",
                      "--no-webserver",
                      action="store_false",
                      dest="webserver",
                      default=True,
                      help="Do not start the Subway web server.")
    parser.add_option("-d",
                      "--daemons",
                      dest="daemons",
                      default="*",
                      help="Specify which daemons to start in a comma-delimited list. Defaults to all daemons. To specify all daemons, do -d *. To specify no daemons, do -d none")
    options, args = parser.parse_args()
    prepare_app(homedir, options.daemons, not options.webserver)
    if options.webserver:
        framework.load_webapp_server(homedir)
    
def wsgi_server(homedir=None):
    """
    start_server() -> wsgiApp : execute a Subway app as WSGI
    assumes homedir is one level up from caller module path unless
    argument is provided.
    """
    import sys
    if not homedir:
        homedir = get_homedir()
    prepare_app(homedir)
    return framework.load_webapp_wsgi(homedir)
